package org.kontalk;


/**
 * isLoggable-aware wrapper around {@link android.util.Log}.
 * @author Daniele Ricci
 */
public final class Log {

    /**
     * Send a {@link #VERBOSE} log message.
     * @param tag Used to identify the source of a log message.  It usually identifies
     *        the class or activity where the log call occurs.
     * @param msg The message you would like logged.
     */
    public static int v(String tag, String msg) {
        if (android.util.Log.isLoggable(tag, android.util.Log.VERBOSE))
            return android.util.Log.v(tag, msg);
        return 0;
    }

    /**
     * Send a {@link #VERBOSE} log message and log the exception.
     * @param tag Used to identify the source of a log message.  It usually identifies
     *        the class or activity where the log call occurs.
     * @param msg The message you would like logged.
     * @param tr An exception to log
     */
    public static int v(String tag, String msg, Throwable tr) {
        if (android.util.Log.isLoggable(tag, android.util.Log.VERBOSE))
            return android.util.Log.v(tag, msg, tr);
        return 0;
    }

    /**
     * Send a {@link #DEBUG} log message.
     * @param tag Used to identify the source of a log message.  It usually identifies
     *        the class or activity where the log call occurs.
     * @param msg The message you would like logged.
     */
    public static int d(String tag, String msg) {
        if (android.util.Log.isLoggable(tag, android.util.Log.DEBUG))
            return android.util.Log.d(tag, msg);
        return 0;
    }

    /**
     * Send a {@link #DEBUG} log message and log the exception.
     * @param tag Used to identify the source of a log message.  It usually identifies
     *        the class or activity where the log call occurs.
     * @param msg The message you would like logged.
     * @param tr An exception to log
     */
    public static int d(String tag, String msg, Throwable tr) {
        if (android.util.Log.isLoggable(tag, android.util.Log.DEBUG))
            return android.util.Log.d(tag, msg, tr);
        return 0;
    }

    /**
     * Send an {@link #INFO} log message.
     * @param tag Used to identify the source of a log message.  It usually identifies
     *        the class or activity where the log call occurs.
     * @param msg The message you would like logged.
     */
    public static int i(String tag, String msg) {
        if (android.util.Log.isLoggable(tag, android.util.Log.INFO))
            return android.util.Log.i(tag, msg);
        return 0;
    }

    /**
     * Send a {@link #INFO} log message and log the exception.
     * @param tag Used to identify the source of a log message.  It usually identifies
     *        the class or activity where the log call occurs.
     * @param msg The message you would like logged.
     * @param tr An exception to log
     */
    public static int i(String tag, String msg, Throwable tr) {
        if (android.util.Log.isLoggable(tag, android.util.Log.INFO))
            return android.util.Log.i(tag, msg, tr);
        return 0;
    }

    /**
     * Send a {@link #WARN} log message.
     * @param tag Used to identify the source of a log message.  It usually identifies
     *        the class or activity where the log call occurs.
     * @param msg The message you would like logged.
     */
    public static int w(String tag, String msg) {
        if (android.util.Log.isLoggable(tag, android.util.Log.WARN))
            return android.util.Log.w(tag, msg);
        return 0;
    }

    /**
     * Send a {@link #WARN} log message and log the exception.
     * @param tag Used to identify the source of a log message.  It usually identifies
     *        the class or activity where the log call occurs.
     * @param msg The message you would like logged.
     * @param tr An exception to log
     */
    public static int w(String tag, String msg, Throwable tr) {
        if (android.util.Log.isLoggable(tag, android.util.Log.WARN))
            return android.util.Log.w(tag, msg, tr);
        return 0;
    }

    /**
     * Send a {@link #WARN} log message and log the exception.
     * @param tag Used to identify the source of a log message.  It usually identifies
     *        the class or activity where the log call occurs.
     * @param tr An exception to log
     */
    public static int w(String tag, Throwable tr) {
        if (android.util.Log.isLoggable(tag, android.util.Log.WARN))
            return android.util.Log.w(tag, tr);
        return 0;
    }

    /**
     * Send an {@link #ERROR} log message.
     * @param tag Used to identify the source of a log message.  It usually identifies
     *        the class or activity where the log call occurs.
     * @param msg The message you would like logged.
     */
    public static int e(String tag, String msg) {
        if (android.util.Log.isLoggable(tag, android.util.Log.ERROR))
            return android.util.Log.e(tag, msg);
        return 0;
    }

    /**
     * Send a {@link #ERROR} log message and log the exception.
     * @param tag Used to identify the source of a log message.  It usually identifies
     *        the class or activity where the log call occurs.
     * @param msg The message you would like logged.
     * @param tr An exception to log
     */
    public static int e(String tag, String msg, Throwable tr) {
        if (android.util.Log.isLoggable(tag, android.util.Log.ERROR))
            return android.util.Log.e(tag, msg, tr);
        return 0;
    }

}
